#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from os.path import join as buildpath
from pentest.manager import JarvisManager
from pentest.utils import JarvisLogger
from datetime import datetime
from slugify import slugify
import shlex
import sys
import subprocess


class JarvisCmd(object):
	"""Top-level Jarvis class that manages penetration tests"""
	def __init__(self, *args):
		self.args = list(*args)
		self.name = os.path.basename(self.args[0])
		self.manager = JarvisManager()
		if self.manager.current:
			self.logger = JarvisLogger(self.manager)
		self.args_config = None
		self.record = False
		self._output_directory = self.manager.config.outdir

		self.clean_env = os.environ.copy()
		dirs = self.clean_env["PATH"].split(':')
		dir_to_exclude = os.path.dirname(self.args[0])
		while dir_to_exclude in dirs:
			del dirs[dirs.index(dir_to_exclude)]
		self.clean_env["SHELL"] = "/bin/bash"
		self.clean_env["PATH"] = ':'.join(dirs)

	def __str__(self):
		return ' '.join(map(lambda x: shlex.quote(x), self.args))

	def __repr__(self):
		return ' '.join(map(lambda x: shlex.quote(x), self.args))

	@property
	def output_directory(self):
		if self.manager.current:
			return buildpath(self.manager.current, self.manager.config.userdir, self._output_directory)
		else:
			return False

	@output_directory.setter
	def output_directory(self, value):
		self._output_directory = value

	def set_args_config(self, args_config):
		self.args_config = args_config

	def set_recordable(self, recordable):
		self.record = recordable

	def run(self, marker=None):
		if not self.manager.current:
			self.args[0] = self.name
			self.execute()
		elif "--nojarvis" in self.args:
			self.args[0] = self.name
			del self.args[self.args.index("--nojarvis")]
			self.execute()
		else:
			for passthrough_arg in self.args_config["passthrough_args"]:
				if passthrough_arg in self.args:
					self.args[0] = self.name
					self.execute()
					break

			self.marker = marker
			self.prepare_command()
			self.logger.register(str(self))
			if self.record:
				JarvisRecordableCommand(self).run()
			else:
				self.execute()

	def prepare_command(self):
		self.args[0] = self.name

		# if no marker is provided, hook will not run
		if self.marker and not self.record:
			for output_arg in self.args_config["output_args"]:
				self.add_arguments(
					output_arg.get("pos", None) or "begin",
					output_arg["arg"],
					output_arg.get("filename", None) or	self.generate_file_name(
						output_arg['format'],
						self.marker
					)
				)

	def generate_file_name(self, format_, marker, record=False):
		filename = self.name + '-'
		filename += (marker.replace('/', '_').replace(':', '_') + '-') if marker else ''
		filename += self.timestamp()
		filename += ('.' + format_) if format_ else ''
		return buildpath(self.output_directory, filename)

	def timestamp(self):
		return slugify(str(datetime.now().replace(microsecond=0)))

	def execute(self):
		"""
		Execute the original program by patching the PATH
		environment variable
		"""
		subprocess.call(self.args, env=self.clean_env)
		sys.exit(0)

	def add_arguments(self, pos, *args):
		"""
		Method used to add safely add arguments into the commandline
		"""
		if pos == "begin":
			for arg in args[::-1]:
				self.args.insert(1, arg)
		elif pos == "end":
			for arg in args:
				self.args.insert(len(self.args), arg)
		else:
			print("[!] Invalid argument position : %s" % pos)
			sys.exit(0)


class JarvisRecordableCommand(object):

	def __init__(self, JarvisCommand):
		self.cmd = JarvisCommand
		self.recordable_cmd = ["script", "-e", "-q", "-c"]
		self.recordable_cmd.append(str(self.cmd))
		self.recordable_cmd.append(self.record_filename)

	@property
	def record_filename(self):
		filename = self.cmd.name + '-'
		filename += "record-"
		filename += (self.cmd.marker.replace('/', '_').replace(':', '_') + '-') if self.cmd.marker else ''
		filename += self.cmd.timestamp()
		filename += ".txt"
		return buildpath(self.cmd.output_directory, filename)

	def run(self):
		subprocess.call(self.recordable_cmd, env=self.cmd.clean_env)
		sys.exit(0)



from pentest.command import JarvisCmd
import sys
__name__ = "nikto"


def main():
	args_config = {
		'passthrough_args': ['-o', '-h', '--help'],
		'output_args': [
			{
				'format': 'txt',
				'arg': '-o'
			}
		]
	}

	cmd = JarvisCmd(sys.argv)
	cmd.set_args_config(args_config)

	if '-host' not in cmd.args:
		cmd.run()

	try:
		target = cmd.args[cmd.args.index('-host') + 1]
	except IndexError:
		cmd.run()
	cmd.run(target)



from pentest.command import JarvisCmd
import sys
__name__ = "nmap"


def main():
	args_config = {
		"passthrough_args": ["-oA", "-oX", "-oG", "-oS", "-oN", "-h", "--help"],
		"output_args": [
			{
				"format": None,
				"arg": "-oA"
			}
		]
	}

	cmd = JarvisCmd(sys.argv)
	cmd.set_args_config(args_config)

	for arg in cmd.args:
		if "--tag" in arg:
			target = cmd.args[cmd.args.index("--tag") + 1]
			del(cmd.args[cmd.args.index("--tag") + 1])
			del(cmd.args[cmd.args.index("--tag")])
			break
		elif "-iL" in arg:
			target = "multiple-hosts"
			break
	else:
		target = cmd.args[-1].replace('/', '_')

	cmd.run(target)



from pentest.command import JarvisCmd
import sys
__name__ = "airodump-ng"


def main():
	args_config = {
		'passthrough_args': ['-w', '-H'],
		'output_args': [
			{
				'format': None,
				'arg': '-w'
			}
		]
	}

	cmd = JarvisCmd(sys.argv)
	cmd.set_args_config(args_config)
	cmd.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pentest.parser import JarvisParser
import uuid
import pkgutil
from glob import glob
from pentest.utils import Config
import subprocess
import json
import os
from os.path import join as buildpath
from os.path import expanduser
import argparse


class JarvisManager(object):
	"""Top-level Jarvis class that manages penetration tests"""
	def __init__(self):
		self.config = Config()
		if not os.path.isfile(self.config.histfile):
			open(self.config.histfile, 'w').close()
		self.fd = open(self.config.histfile, 'r+')

		self.update()
		self.parser = argparse.ArgumentParser()
		subparsers = self.parser.add_subparsers(dest='command', help='Action to perform')
		start_parser = subparsers.add_parser('start', help='Start the pentest in the provided directory')
		start_parser.add_argument('directory', help='The directory name')
		subparsers.add_parser('stop', help='Stop the current pentest')
		subparsers.add_parser('init', help='Creates working tree for current pentest')
		subparsers.add_parser('screenshot', help='Takes a screenshot')
		edit_parser = subparsers.add_parser('edit', help='Runs image editor on a screenshot')
		edit_parser.add_argument('image', help='The screenshot to edit (filename)')
		subparsers.add_parser('notes', help='Starts notes edition')
		subparsers.add_parser('img', help='Lists screenhosts')
		subparsers.add_parser('hooks', help='Lists available hooks')

	def update(self):
		self.fd.seek(0, os.SEEK_END)
		if self.fd.tell() == 0:
			self.current = None
		else:
			while True:
				self.fd.seek(self.fd.tell() - 1, os.SEEK_SET)
				char = self.fd.read(1)
				self.fd.seek(self.fd.tell() - 1, os.SEEK_SET)
				if char == "\n" or self.fd.tell() == 0:
					if char == "\n":
						self.fd.read(1)
					self.current = self.fd.read()
					break
		self.fd.seek(0, os.SEEK_SET)

	def shortcutize(self):
		try:
			os.unlink("%s/pentest" % expanduser("~"))
		except Exception:
			pass
		if self.current:
			os.symlink(self.current, "%s/pentest" % expanduser("~"))

	def start(self, path):
		self.fd.seek(0, os.SEEK_END)
		if os.path.isdir(path):
			if os.path.getsize(self.config.histfile) == 0:
				self.fd.write("%s" % os.path.abspath(path))
			else:
				self.fd.write("\n%s" % os.path.abspath(path))
		self.fd.seek(0, os.SEEK_SET)
		self.current = os.path.abspath(path)
		self.shortcutize()
		print("[i] A pentest located at %s has been stacked" % os.path.abspath(path))

	def stop(self):
		self.fd.seek(0, os.SEEK_END)
		if self.fd.tell() != 0:
			while True:
				self.fd.seek(self.fd.tell() - 1, os.SEEK_SET)
				char = self.fd.read(1)
				self.fd.seek(self.fd.tell() - 1, os.SEEK_SET)
				if char == '\n' or self.fd.tell() == 0:
					self.fd.truncate()
					break
		self.fd.seek(0, os.SEEK_SET)
		self.update()
		self.shortcutize()

	def init(self):
		if self.current:
			for directory in self.config.tree:
				if not os.path.isdir(os.path.join(self.current, directory)):
					os.mkdir(os.path.join(self.current, directory))
				else:
					print("[!] %s already exists, skipping" % os.path.join(self.current, directory))

	def screenshot(self):
		screenshot_id = str(uuid.uuid4())
		subprocess.check_output(["import", buildpath(self.current, self.config.userdir, self.config.imgdir, "%s.png" % screenshot_id)])
		screenshot_name = subprocess.check_output(["zenity", "--entry", "--title=Jarvis-Screen", "--text=Screenshot name"])
		screenshot_name = screenshot_name[:-1]
		while True:
			if os.path.isfile(os.path.join(self.current, self.config.userdir, self.config.imgdir, "%s.png" % screenshot_name)):
				screenshot_name = subprocess.check_output(["zenity", "--entry", "--title=Jarvis-Screen", "--text=Name already taken"])
				screenshot_name = screenshot_name[:-1]
			else:
				break

		os.rename(
			os.path.join(self.current, self.config.userdir, self.config.imgdir, "%s.png" % screenshot_id),
			os.path.join(self.current, self.config.userdir, self.config.imgdir, "%s.png" % screenshot_name)
		)

	def notes(self):
		subprocess.call([self.config.editor, buildpath(self.current, self.config.userdir, self.config.notes)])

	def hooks(self):
		jarvis_path = os.path.dirname(__import__("pentest").__file__)
		hooks_path = buildpath(jarvis_path, "hooks")
		for _, name, _ in pkgutil.iter_modules([hooks_path]):
			print(name)

	def img(self):
		imgs_paths = glob("%s/*" % buildpath(self.current, self.config.userdir, self.config.imgdir))
		imgs_filenames = map(lambda x: os.path.basename(x), imgs_paths)
		for img_filename in imgs_filenames:
			print(img_filename)

	def edit(self, img):
		img_path = buildpath(self.current, self.config.userdir, self.config.imgdir, img)
		if os.path.isfile(img_path):
			subprocess.call(["pinta", img_path])
		else:
			print("[!] This image does not exists")



import subprocess
import inspect
import os
import sys


def add_arguments(cmd, *args):
	for arg in args[::-1]:
		cmd.insert(1, arg)


def execute_without_jarvis(args):
	"""
	Execute the original program by patching the PATH
	environment variable
	"""
	jarvis_env = os.environ.copy()
	dirs = jarvis_env['PATH'].split(':')
	script_path = os.path.dirname(sys.argv[0])
	dir_to_exclude = os.path.abspath(script_path)
	del dirs[dirs.index(dir_to_exclude)]
	jarvis_env['PATH'] = ':'.join(dirs)
	args[0] = os.path.basename(sys.argv[0])
	subprocess.call(args, env=jarvis_env)
	sys.exit(0)



from pentest.command import JarvisCmd
import sys

__name__ = "dnsrecon"


def main():
	args_config = {
		'passthrough_args': ['-c', '--csv', '-j', '--json', '--xml', '-h', '--help'],
		'output_args': [
			{
				'format': 'json',
				'arg': '-j'
			},
			{
				'format': 'csv',
				'arg': '-c'
			},
			{
				'format': 'xml',
				'arg': '--xml'
			}
		]
	}

	cmd = JarvisCmd(sys.argv)
	cmd.set_args_config(args_config)

	if '-d' not in cmd.args:
		cmd.run()

	try:
		target = cmd.args[cmd.args.index('-d') + 1]
	except IndexError:
		cmd.run()
	cmd.run(target)



from pentest.command import JarvisCmd
import sys
__name__ = "crackmapexec"


def main():
	args_config = {
		'passthrough_args': ['-h', '--help'],
	}

	cmd = JarvisCmd(sys.argv)
	cmd.set_recordable(True)
	cmd.set_args_config(args_config)
	cmd.run()



import sys
import os
from os.path import join as buildpath
from pentest import __version__
from pentest.parser import JarvisParser
from pentest.manager import JarvisManager


def main(args=None):
	"""
	Main entry point for Jarvis

	Creates a user-level .jarvis directory for managing a couple of configuration
	options
	"""
	if not os.path.isfile("/etc/jarvis.conf"):
		print("[!] File /etc/jarvis.conf does not exist, please create it")
		print("Sample configuration file below, please make sure the pentest_history file will be writable by Jarvis users")
		print()
		print("[jarvis]")
		print("pentests_history = /home/marauder/.pentests_history")
		print("user_directory = user")
		print("output_directory = records")
		print("pentest_tree = [")
		print('	"%(user_directory)s",')
		print('	"%(user_directory)s/img",')
		print('	"%(user_directory)s/scripts",')
		print('	"%(user_directory)s/records"')
		print("	]")
		print("notesfiles = notes.txt")
		print("interface = eth0")
		print("editor = vim")

		sys.exit(0)

	manager = JarvisManager()

	# Execute user-supplied action
	args = manager.parser.parse_args()
	if args.command == "start":
		manager.start(args.directory)

	if manager.current:
		if args.command == "edit":
			manager.edit(args.image)
		elif args.command == "stop":
			manager.stop()
		elif args.command == "init":
			manager.init()
		elif args.command == "screenshot":
			manager.screenshot()
		elif args.command == "notes":
			manager.notes()
		elif args.command == "img":
			manager.img()
		elif args.command == "hooks":
			manager.hooks()
	else:
		print("[!] Create a pentest directory first")


if __name__ == "__main__":
	main()



import logging
import json
import sys
from logging import FileHandler
from os.path import join as buildpath
import netifaces as ni
from configparser import NoOptionError, ConfigParser

COMMAND_LEVEL = 99

# add a new logging level dedicated to history
logging.addLevelName(COMMAND_LEVEL, 'COMMAND_LEVEL')


def history(self, message, *args, **kwargs):
	if self.isEnabledFor(COMMAND_LEVEL):
		self._log(COMMAND_LEVEL, message, args, **kwargs)


logging.Logger.history = history


class CommandFilter(object):
	def __init__(self, level):
		self.__level = level

	def filter(self, logRecord):
		return logRecord.levelno == self.__level


class CommandHistoryFormatter(logging.Formatter):
	def __init__(self, fmt):
		super(CommandHistoryFormatter, self).__init__(fmt)

	def format(self, record):
		config = Config()
		try:
			ip = ni.ifaddresses(config.iface)[2][0]['addr']
			record.emitter = ip
		except Exception:
			record.emitter = "unavailable"
		return super(CommandHistoryFormatter, self).format(record)


class JarvisLogger(object):

	def __init__(self, manager):
		self.logger = logging.getLogger()
		self.logger.addFilter(CommandFilter(COMMAND_LEVEL))
		formatter = CommandHistoryFormatter('[%(emitter)s] %(asctime)s :: %(message)s')

		file_handler = FileHandler(
			buildpath(
				manager.current,
				manager.config.userdir,
				'commands.log'
			),
			mode='a',
			encoding='utf-8',
			delay=0
		)
		file_handler.addFilter(CommandFilter(COMMAND_LEVEL))
		file_handler.setFormatter(formatter)
		self.logger.addHandler(file_handler)

	def register(self, command):
		self.logger.history(command)


class Config(object):

	def __init__(self):
		try:
			config = ConfigParser()
			config.read("/etc/jarvis.conf")
			self.userdir = config.get('jarvis', 'user_directory')
			self.outdir = config.get('jarvis', 'output_directory')
			self.imgdir = config.get('jarvis', 'img_directory')
			self.histfile = config.get('jarvis', 'pentests_history')
			self.tree = json.loads(config.get('jarvis', 'pentest_tree'))
			self.notes = config.get('jarvis', 'notesfiles')
			self.iface = config.get('jarvis', 'interface')
			self.editor = config.get('jarvis', 'editor')
		except NoOptionError as e:
			print(e)
			sys.exit(0)
